import json
import resonances


def create_simulation_from_json(json_file_src):
    with open(json_file_src, "r") as read_file:
        c_config = json.load(read_file)

    sim = resonances.Simulation()
    sim.setup(
        save=c_config['save'],
        plot=c_config['plot'],
        save_path=c_config['save_path'],
    )
    sim.create_solar_system()

    if 'integration.Nout' in c_config:
        sim.Nout = int(c_config['integration.Nout'])
    if 'integration.tmax' in c_config:
        sim.tmax = int(c_config['integration.tmax'])
    if 'integration.dt' in c_config:
        sim.dt = c_config['integration.dt']

    # @todo need to verify that data are full
    # Add checks for asteroids, resonances, Nout and stop, plot (or default values)
    for asteroid in c_config['asteroids']:
        if 'num' in asteroid['elem']:
            elem_or_num = asteroid['elem']['num']
        else:
            elem_or_num = asteroid['elem']
        # @todo validation
        for resonance in asteroid['resonances']:
            sim.add_body(
                elem_or_num, resonances.create_mmr(resonance['integers'], resonance['bodies']), '{}'.format(asteroid['elem']['label'])
            )

    return sim



import json
import time
import argparse

import resonances
from resonances.matrix.three_body_matrix import ThreeBodyMatrix
from resonances.matrix.two_body_matrix import TwoBodyMatrix
from resonances.data.astdys import astdys

from resonances.experiment import shape
from resonances.experiment import loader
from resonances.experiment import finder

parser = argparse.ArgumentParser(description='')


def get_parser():
    return argparse.ArgumentParser(description='')


def quick():
    parser.add_argument('asteroid', help='The number of the asteroid you want to research.', type=int)
    parser.add_argument('resonance', help='The resonance in a short notation like 4J-2S-1 or 2J-1.', type=str)
    args = parser.parse_args()

    resonances.logger.info('Starting simulation "quick" for the asteroid {} and the resonance {}.'.format(args.asteroid, args.resonance))
    sim = resonances.Simulation()
    sim.create_solar_system()
    sim.add_body(args.asteroid, resonances.create_mmr(args.resonance), 'ast-{}'.format(args.asteroid))
    sim.run()
    resonances.logger.info('Successfully ended.')


def asteroids():
    parser.add_argument('--config', nargs='?', type=str)
    args = parser.parse_args()

    sim = loader.create_simulation_from_json(args.config)
    resonances.logger.info('Starting simulation "asteroids" based on the config {}'.format(args.config))
    sim.run()
    resonances.logger.info('Successfully ended.')


def calc_shape():
    start_time = time.time()
    parser.add_argument('--config', nargs='?', type=str)
    args = parser.parse_args()

    with open(args.config, "r") as read_file:
        c_config = json.load(read_file)

    mmr_template = resonances.create_mmr(c_config['resonance']['integers'], c_config['resonance']['bodies'])
    resonances.logger.info('Starting shape simulation based on the file {}'.format(args.config))
    shape.run(
        c_config['elem'],
        c_config['variations'],
        mmr_template,
        c_config['save'],
        c_config['save_path'],
        c_config['plot'],
        c_config['dump'],
    )

    resonances.logger.info("Successfully ended. --- %s seconds ---" % (time.time() - start_time))


def identifier():
    start_time = time.time()
    parser.add_argument('asteroid', help='The number of the asteroid you want to research.', type=int)
    parser.add_argument(
        '--planets', help='The planets, whose three body or two body resonances are searched. Use comma to separate planets.', type=str
    )
    args = parser.parse_args()

    resonances.config.set('save.path', 'cache/identifier')
    resonances.logger.info('Starting simulation "identifier" for the asteroid {}'.format(args.asteroid))

    planets = None
    if args.planets is not None:
        planets = [item.strip() for item in args.planets.split(',')]

    sim = resonances.Simulation()
    sim.create_solar_system()
    elem = astdys.search(args.asteroid)
    mmrs = ThreeBodyMatrix.find_resonances(elem['a'], planets=planets)
    mmrs2 = TwoBodyMatrix.find_resonances(elem['a'], planets=planets)
    mmrs = mmrs + mmrs2
    resonances.logger.info('The asteroid {} is found.'.format(args.asteroid))
    resonances.logger.info('The value of semi-major axis is {:6.4f}'.format(elem['a']))
    for mmr in mmrs:
        resonances.logger.info('Adding a possible resonance: {}'.format(mmr.to_short()))
        sim.add_body(args.asteroid, mmr, '{}-{}'.format(args.asteroid, mmr.to_short()))
    resonances.logger.info('Running integration.')
    sim.run()
    resonances.logger.info("Successfully ended. --- %s seconds ---" % (time.time() - start_time))


def asteroids_in_resonance():
    parser.add_argument('resonance', help='The resonance in a short notation like 4J-2S-1.', type=str)
    parser.add_argument(
        '--iterations',
        help='The maximum number of iterations. One iteration consists of 100 asteroid candidates. Should be integer.',
        type=int,
    )
    args = parser.parse_args()

    iterations = 1000
    if args.iterations is not None:
        iterations = args.iterations

    resonances.logger.info('Starting simulation "asteroids in resonance" for the resonance {}'.format(args.resonance))
    resonances.logger.info('The number of iterations: {}'.format(iterations))
    finder.run(resonances.create_mmr(args.resonance), dump=100, max_iterations=iterations)
    resonances.logger.info('Successfully ended.')



import pandas as pd
import numpy as np
from pathlib import Path
import urllib.request

import resonances.config
import resonances.logger


class astdys:

    catalog = None

    @classmethod
    def search(cls, num):
        num = str(num)
        if cls.catalog is None:
            cls.load()

        if num in cls.catalog['num'].values:
            return cls.catalog.loc[cls.catalog['num'] == num].to_dict('records')[0]

        return None

    @classmethod
    def search_possible_resonant_asteroids(cls, mmr, sigma=0.02):
        if cls.catalog is None:
            cls.load()
        axis = mmr.resonant_axis
        df = cls.catalog[(cls.catalog['a'] >= axis - sigma) & (cls.catalog['a'] <= axis + sigma)]
        return df

    @classmethod
    def load(cls):
        if cls.catalog is None:
            output_file = Path(resonances.config.get('catalog'))
            if not output_file.exists():
                cls.build()

        cls.catalog = pd.read_csv(resonances.config.get('catalog'))
        cls.catalog['num'] = cls.catalog['num'].astype(str)

    @classmethod
    def build(cls):
        input_file = Path(resonances.config.get('astdys.catalog'))
        if not input_file.exists():
            resonances.logger.info('Cannot find AstDyS catalog. Trying to download it...')
            try:
                urllib.request.urlretrieve(resonances.config.get('astdys.catalog.url'), 'cache/allnum.cat')
            except Exception:
                raise Exception(
                    "No input catalog available. Cannot download it too. Put AstDys allnum.cat or allnum.csv in the cache directory!"
                )
            resonances.logger.info('Successfully downloaded. Continue working...')

        cat = cls.transform_astdys_catalog()
        cat.to_csv(resonances.config.get('catalog'), index=False)

    @classmethod
    def transform_astdys_catalog(cls):
        catalog = pd.read_csv(resonances.config.get('astdys.catalog'), delim_whitespace=True, skiprows=5)
        cat = catalog.rename(
            columns={
                '!': 'num',
                'Name,': 'epoch',
                'Epoch(MJD),': 'a',
                'a,': 'e',
                'e,': 'inc',
                'i,': 'Omega',
                'long.': 'omega',
                'node,': 'M',
            }
        )
        cat['num'] = cat['num'].str.replace("'", "")
        deg_cols = ['inc', 'Omega', 'omega', 'M']
        for col in deg_cols:
            cat[col] = cat[col].map(lambda x: float(x) * np.pi / 180)

        cat.drop(cat.columns[[1, 8, 9, 10, 11]], axis=1, inplace=True)
        return cat



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['resonances',
 'resonances.data',
 'resonances.experiment',
 'resonances.matrix',
 'resonances.resonance']

package_data = \
{'': ['*']}

install_requires = \
['astropy>=5.2.1,<6.0.0',
 'matplotlib>=3.6.3,<4.0.0',
 'numpy>=1.24.1,<2.0.0',
 'pandas>=1.5.3,<2.0.0',
 'rebound>=3.23.2,<4.0.0',
 'scipy>=1.10.0,<2.0.0',
 'seaborn>=0.12.2,<0.13.0']

entry_points = \
{'console_scripts': ['ain = '
                     'resonances.experiment.console:asteroids_in_resonance',
                     'asteroids-in-resonance = '
                     'resonances.experiment.console:asteroids_in_resonance',
                     'ia = resonances.experiment.console:asteroids',
                     'identify-asteroids = '
                     'resonances.experiment.console:asteroids',
                     'identify-quick = resonances.experiment.console:quick',
                     'identify-resonances = '
                     'resonances.experiment.console:identifier',
                     'iq = resonances.experiment.console:quick',
                     'ir = resonances.experiment.console:identifier',
                     'simulation-shape = '
                     'resonances.experiment.console:calc_shape',
                     'ss = resonances.experiment.console:calc_shape']}

setup_kwargs = {
    'name': 'resonances',
    'version': '0.2.3',
    'description': 'Identification of mean-motion resonances',
    'long_description': '# Mean-Motion Resonances\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Resonances](https://github.com/smirik/resonances/actions/workflows/ci.yml/badge.svg?branch=main)](https://github.com/smirik/resonances/actions/workflows/ci.yml)\n\n`resonances` is an open-source package dedicated to the identification of mean-motion resonances of small bodies. Many examples are for the Solar system; however, you might use the package for any possible planetary system, including exoplanets.\n\nFor more information, [read the documentation](https://smirik.github.io/resonances/).\n\n**Note:** while this app has many functional and integration tests built in, it is still in the dev stage. Hence, it might include some inconsistencies. So, any community help is appreciated!\n\n## Features\n\nThe package:\n\n- can automatically identify two-body and three-body mean-motion resonance in the Solar system,\n- accurately differentiates different types of resonances (pure, transient, uncertain),\n- provides an interface for mass tasks (i.e. find resonant areas in a planetary system),\n- can plot time series and periodograms,\n- and, yeah, it is well tested ;)\n\nIt actively uses [REBOUND integrator](https://rebound.readthedocs.io) maintained by Hanno Rein and others.\n\n## Installation\n\nTo install resonances on your system, follow the instructions on the appropriate [installation guide](https://smirik.github.io/resonances/install/)\n\n## Mean-motion resonances\n\nFor those who are not familiar with the mean-motion resonances, here is the list of papers used to develop this package:\n\n### Papers about the automatic identification of resonant asteroids\n\n1. Smirnov, E. A. & Shevchenko, I. I. Massive identification of asteroids in three-body resonances. Icarus 222, 220–228 (2013).\n1. Smirnov, E. A., Dovgalev, I. S. & Popova, E. A. Asteroids in three-body mean motion resonances with planets. Icarus (2017) doi:10.1016/j.icarus.2017.09.032.\n1. Smirnov, E. A. & Dovgalev, I. S. Identification of Asteroids in Two-Body Resonances. Solar System Research 52, 347–354 (2018).\n1. Nesvorný, D. & Morbidelli, A. Three-Body Mean Motion Resonances and the Chaotic Structure of the Asteroid Belt. The Astronomical Journal 116, 3029–3037 (1998).\n\n### Papers about mean-motion resonances\n\n1. Chirikov, B. V. A universal instability of many-dimensional oscillator systems. Physics reports 52, 263–379 (1979).\n1. Gallardo, T. Strength, stability and three dimensional structure of mean motion resonances in the solar system. Icarus 317, 121–134 (2019).\n1. Gallardo, T. Atlas of the mean motion resonances in the Solar System. Icarus 184, 29–38 (2006).\n1. Gallardo, T., Coito, L. & Badano, L. Planetary and satellite three body mean motion resonances. Icarus 274, 83–98 (2016).\n1. Milani, A., Cellino, A., Knezevic, Z., Novaković, B. & Spoto, F. Asteroid families classification: Exploiting very large datasets. Icarus 239, 46–73 (2014).\n1. Murray, N. & Holman, M. Diffusive chaos in the outer asteroid belt. The Astronomical Journal 114, 1246 (1997).\n1. Murray, N., Holman, M. & Potter, M. On the Origin of Chaos in the Asteroid Belt. The Astronomical Journal 116, 2583–2589 (1998).\n1. Shevchenko, I. I. On the Lyapunov exponents of the asteroidal motion subject to resonances and encounters. Proc. IAU 2, 15–30 (2006).\n\n### Books\n\n1. Murray, C. D. & Dermott, S. F. Solar system dynamics. (Cambridge Univ. Press, 2012).\n1. Morbidelli, A. Modern celestial mechanics: aspects of solar system dynamics. (2002).\n\n## References\n\nWhenever you use this package, we are kindly asking you to refer to one of the following papers (please choose the appropriate):\n\n1. **The package itself**: to be published\n1. **The Libration module and automatic identification of librations**: to be published\n1. **Mass identification of mean-motion resonances:** Smirnov, E. A., Dovgalev, I. S. & Popova, E. A. Asteroids in three-body mean motion resonances with planets. Icarus (2017) doi:10.1016/j.icarus.2017.09.032.\n\n## Authors\n\nThe authors of the package:\n\n- [Evgeny Smirnov](https://github.com/smirik) ([FB](https://facebook.com/smirik), [Telegram](https://t.me/smirik))\n\n## Acknowledgement\n\n- Many thanks to the co-authors of the papers (prof. I.\xa0I. Shevchenko, I.\xa0Dovgalev Dr.\xa0E.\xa0Popova).\n- The creators of [REBOUND integrator](https://rebound.readthedocs.io).\n- The creators of [Astropy](http://astropy.org).\n- The creators of `numpy`, `scipy`, `pandas`, and `matplotlib`.\n\n## Contributing\n\nFeel free to contribute to the code by sending pull requests [to the repository](https://github.com/smirik/resonances).\n\n## License\n\nMIT\n',
    'author': 'Evgeny Smirnov',
    'author_email': 'smirik@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'http://smirik.github.io/resonances/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<3.12',
}


setup(**setup_kwargs)



import numpy as np
import math
from itertools import product

import resonances


def run(elem, variations, mmr: resonances.MMR, save=False, save_path="cache", need_plot=False, dump=100):

    particles = []
    for variation in variations:
        a_arr = np.linspace(variation['a']['min'], variation['a']['max'], variation['a']['num'])
        e_arr = np.linspace(variation['e']['min'], variation['e']['max'], variation['e']['num'])
        particles += list(product(a_arr, e_arr))

    num_particles = len(particles)

    num_iterations = int(math.ceil(num_particles / dump))

    for j in range(num_iterations):
        sim = resonances.Simulation(save=save, save_path=save_path, plot=need_plot)
        sim.create_solar_system()
        # Find the number of calculations for a given step. It varies for the last one.
        if j < num_iterations - 1:
            num = dump
        else:
            num = num_particles % dump
            if num == 0:
                num = dump

        ae_data = np.zeros((num, 4))

        for i in range(num):
            key = j * dump + i
            elem = {
                'a': particles[key][0],
                'e': particles[key][1],
                'inc': elem['inc'],
                'Omega': elem['Omega'],
                'omega': elem['omega'],
                'M': elem['M'],
            }
            sim.add_body(elem, mmr, '{}'.format(key))

            ae_data[i][0] = key
            ae_data[i][1] = particles[key][0]
            ae_data[i][2] = particles[key][1]

        sim.run()

        if save:
            for i, body in enumerate(sim.bodies):
                ae_data[i][3] = body.status
            with open('{}/ae-plane.csv'.format(save_path), 'a') as f:
                np.savetxt(f, ae_data, delimiter=',')

        resonances.logger.info('Iteration {} has finished. Processed from {}. Starting the new one.'.format(j, j * dump))
